/* configuration procedures for PipBase */

#include <stdio.h>
#include <alloc.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>
#include <ctype.h>
#include <io.h>
#include <dir.h>


#include "pipbase.h"

#undef MAIN

#include "pipext.h"


dirset(char *dest,char *src)
{
  int i;
  struct ffblk blk;
  if ((i=strlen(src))>63)
    {fprintf(stderr,"Directory name \"%s\" is too long!\n\7",src); exit(2);}
  strcpy(dest,src); if (dest[i-1]!='\\') {dest[i]='\\'; dest[i+1]=0;}
  if(src[i-1]=='\\')
    src[i-1]=0;
  if(findfirst(src,&blk,FA_DIREC) && (strlen(src)>2))
    fprintf(stderr,"Warning: directory %s does not exist; please create it.\n",src);
  strupr(dest);
}


write_nodeinfo()
{
  f=fopen("NodeInfo.Cfg","rb+");
  if (f==NULL)
    f=fopen("NodeInfo.Cfg","wb");    
  if (f==NULL)
    esci("I cannot create NodeInfo.Cfg!");
  printf("Writing NodeInfo.Cfg...\n");
  fwrite(&cfg,sizeof cfg,1,f);
  fclose(f);
}


write_areas()
{
  uint i;
  sprintf(fn,"%sBaseDesc.Pip",cfg.pipdir); f=fopen(fn,"rb+");
  if (f==NULL) f=fopen(fn,"wb");
  if (f==NULL)
    {
      fprintf(stderr,"I cannot create the PipBase files; please check for %s\7\n",fn);
      exit(2);
    }
  printf("Writing %s...\n",fn);
  for(i=0; i!=cfg.pipsize; i++) fwrite(&areas[i],sizeof(AREASTRUCT),1,f);
  fclose(f);
}


read_cfg(int l)
{
  int i;
  if ((l==1) || (l==0))
    {
      f=fopen("NodeInfo.Cfg","rb");
      if (f==NULL)
        {
          fprintf(stderr,"I cannot find NODEINFO.CFG file; please run PIPBASE SET ....\7\n");
          exit(2);
        }
      fread(&cfg,sizeof cfg,1,f);
      fclose(f);
      areas=(AREASTRUCT *)calloc(cfg.pipsize,sizeof(AREASTRUCT));
      if (areas==NULL)
        {
          fprintf(stderr,"I cannot store area definition... what you need is more memory... or less areas\7\n");
          exit(2);
        }
    }
  if ((l==2) || (l==0))
    {
      sprintf(fn,"%sOrigins.Pip",cfg.pipdir); f=fopen(fn,"rb");
      if (fn==NULL)
        {
          fprintf(stderr,"I cannot locate the PipBase files; please check for %s\7\n",fn);
          exit(2);
        }
      fread(origins,sizeof(origins),1,f); fclose(f);
    }
  if ((l==3) || (l==0))
    {
      sprintf(fn,"%sBaseDesc.Pip",cfg.pipdir); f=fopen(fn,"rb");
      if (fn==NULL)
        {
          fprintf(stderr,"I cannot locate the PipBase files; please check for %s\7\n",fn);
          exit(2);
        }
      for (i=0; i!=cfg.pipsize; i++)
        fread(&areas[i],sizeof(AREASTRUCT),1,f);
      fclose(f);
    }
}


addroute(uint via,uint zone,uint net,uint node,uint point,uint mode)
{
  ROUTE r;
  r.via=via; r.zone=zone; r.net=net; r.node=node; r.point=point; r.mode=mode;
  sprintf(fn,"%sRoute.Pip",cfg.pipdir);
  if((f=fopen(fn,"ab"))==NULL)
    if((f=fopen(fn,"wb"))==NULL)
      {
        sprintf(lbuff,"Cannot create routing file %s",fn);
        esci(lbuff);
      }
  fwrite(&r,sizeof r,1,f);
  printf("Rule #%d added.\n",ftell(f)/sizeof(r)-1);
  fclose(f);
}


delroute(uint rule)
{
  char fn1[80],fn2[80];
  int rn;
  FILE *f1,*f2;
  ROUTE r;
  rn=rule;
  sprintf(fn1,"%sRoute.Pip",cfg.pipdir); f1=fopen(fn1,"rb");
  if(!f1) esci("No route rule exist!");
  sprintf(fn2,"%sRoute.Pi2",cfg.pipdir); f2=fopen(fn2,"wb");
  while (fread(&r,sizeof r,1,f1))
    if(rn) {fwrite(&r,sizeof r,1,f2); rn--;} else rn--;
  if (rn>0) printf("Rule#%d does not exist - run PIPBASE LISTROUTE\n",rule);
  fclose(f1); fclose(f2);
  if(rn>0)
    {unlink(fn2); return;}
  if(filelength(fileno(f2))==0)
    {
      fclose(f2);
      unlink(fn1); unlink(fn2);
      printf("No more route defined\n");
    }
  else
    {
      fclose(f2);
      unlink(fn1); rename(fn2,fn1);
      printf("OK: rule deleted - other rules may have been renumbered\n");
    }
}


listareas(int verbose)
{
  uint i;
  printf("Message areas:\n");
  for (i=0; i!=cfg.pipsize; i++)
    if (*areas[i].descr)
      {
        printf("%5d : ",i);
        printf(areas[i].descr);
        printf(" (Tag: %s)",areas[i].tag);
        if (verbose)
          printf("\n       %d msgs, %d days, %s, origin#%d, address#%d,\n       read at level %d (flags=%1x%1x%1x%1x), write at %d (flags=%1x%1x%1x%1x)\n",
                 areas[i].nrmsgs,areas[i].days,areas[i].killrcv?"Kill/Rcvd":"Leave/Rcvd",
                 areas[i].origin,areas[i].address,
                 areas[i].readlevel,areas[i].readflags[0],areas[i].readflags[1],areas[i].readflags[2],areas[i].readflags[3],
                 areas[i].writelevel,areas[i].writeflags[0],areas[i].writeflags[1],areas[i].writeflags[2],areas[i].writeflags[3]);
        printf("\n");
      }
}


listcfg()
{
  char *attstyle[3]={"Pip!","FrontDoor/D'Bridge","Binkley/TrapDoor"};
  char *pack[2]={"Dynamic Packing","Static Packing"};
  printf("Configured directories:\n");
  printf("PipBase    : %s (Size: %s, %d message areas)\n",cfg.pipdir,(cfg.pipsize==64)?"TINY":((cfg.pipsize==256)?"SMALL":((cfg.pipsize==1024)?"MEDIUM":((cfg.pipsize==4096)?"LARGE":((cfg.pipsize==32768)?"EXTRA-LARGE":"???")))),cfg.pipsize);
  printf("InBound    : %s\nOutBound   : %s\n",cfg.inbound,cfg.outbound);
  printf("Packers    : %s\nTemporary  : %s\n",cfg.packerdir,cfg.tempdir);
  printf("Packer     : %s %s\n",cfg.packer,cfg.packerparms);
  printf("Attaches   : %s (%s, %s)\n",cfg.attaches,attstyle[cfg.attachstyle],pack[cfg.pack_matrix]);
  printf("Log file   : %s (%s level %d)\n",cfg.logfilename,cfg.logtype?"FroDo/D'B":"Bink/Opus",cfg.loglevel);
  printf("Remove     : %s%s%s%s%s\n",
         (cfg.compact&SET_COMPACT_REMOVE_KLUDGES)?"Kludges ":"",
         (cfg.compact&SET_COMPACT_REMOVE_REPLY)?"Re: ":"",
         (cfg.compact&SET_COMPACT_REMOVE_4D)?"4D-info ":"",
         (cfg.compact&SET_COMPACT_KILLROBOTRQ)?"Robot requests ":"",
         (cfg.compact&SET_COMPACT_KILLROBOTASW)?"Robot answers ":"");
  printf("Ghostwriter: %s\n",*cfg.ghost?cfg.ghost:"PipBase GhostWriter");
  if(*cfg.receipt)
    printf("Ret Receipt: %s\n",cfg.receipt);
  if(registered) printf("Registered : #%d (%s)\n",registered,cfg.key);
  printf("See also LISTAREAS, LISTUSERS, LISTAKAS, LISTFRIENDS and LISTORIGINS\n");
}


listexport()
{
  uint i,j;
  FRIENDND fri;
  sprintf(fn,"%sFriendNd.Pip",cfg.pipdir); f=fopen(fn,"rb");
  for (i=0; i!=cfg.pipsize; i++)
    if (*areas[i].tag)
      {
        printf("%5d: %-30s ",i,areas[i].tag);
        for (j=0; j!=256; j++)
          if(isbit(j,areas[i].forward_to))
          {
            if(fseek(f,j*sizeof fri,SEEK_SET))
              printf("UNKNOWN NODE %d! ",j);
            else
              {
                fread(&fri,sizeof fri,1,f);
                printf("%d:%d/%d.%d ",fri.zone,fri.net,fri.node,fri.point);
              }
          }
        printf("\n");
      }
  fclose(f);
}


listfriends()
{
  FRIENDND fri;
  int i=0;
  sprintf(fn,"%sFriendNd.Pip",cfg.pipdir); f=fopen(fn,"rb");
  while (fread(&fri,sizeof fri,1,f))
    {
      if (fri.flags&FRI_SET_ACTIVE) /* if active */
         {
           printf("%4d: %d:%d/%d.%d %s\n",i,fri.zone,fri.net,fri.node,fri.point,(fri.flags&1)?"Uses Pipbase":"");
           printf("      Packer: %s %s%s %s\n",fri.packer,fri.packerparms,(*fri.pack_pw)?", passworded with ":"",fri.pack_pw);
           if (fri.areafix_lev) printf("      Areafix level: %d (Password %s)\n",fri.areafix_lev,fri.areafix_pw);
           printf("      %s\n",fri.notes);
         }
       i++;
     }
  fclose(f);
}


listorigins()
{
  int i;
  printf("Origins:\n");
  for (i=0; i!=256; i++) if (*origins[i]) printf("%3d : %s\n",i,origins[i]);
}


listroute()
{
  uint i;
  FRIENDND fri;
  FILE *ff;
  ROUTE route;
  sprintf(fn,"%sFriendNd.Pip",cfg.pipdir);
  if ((ff=fopen(fn,"rb"))==NULL)
    esci("Route hasn't any sense without friends! Please run PIPBASE SETFRIEND");
  sprintf(fn,"%sRoute.Pip",cfg.pipdir);
  if((f=fopen(fn,"rb"))==NULL)
    {
      printf("No special route is specified.\nConsidering your role of ");
      if(cfg.address[0].point)
        {
          printf("point, all your mail will be routed via your boss\n");
          while(fread(&fri,sizeof fri,1,ff))
            if((fri.zone==cfg.address[0].zone) &&
               (fri.net==cfg.address[0].net) &&
               (fri.node==cfg.address[0].node) &&
               !fri.point)
              {
                printf("%d:%d/%d.0, friend#%d",fri.zone,fri.net,fri.node,ftell(ff)/sizeof(fri)-1);
                fclose(ff);
                return;
              }
          else
            {
              printf("but your boss isn't between your friends! set it, or choose a special route!\n\7"); 
              fclose(ff);
              return;
            }
        }
      else
        {
          if(!cfg.address[0].node)
            {
              printf("host, all mail for your subnodes will be sent\ndirectly; mail for other nets will be routed via their hosts.\n");
              printf("Be sure that all them are between your friends\n");
              fclose(ff); return;
            }
          else
            {
              printf("normal BBS, all mail for your points will be\nheld, and all other mail will be routed via your host.\n");
              printf("Be sure that your host and your points are between your friends!\n");
              fclose(ff); return;
            }
        }
    }
  i=0;
  while(fread(&route,sizeof route,1,f))
    {
      fseek(ff,route.via*sizeof(fri),SEEK_SET);
      fread(&fri,sizeof(fri),1,ff);
      printf("%5d ROUTE VIA %d [%d:%d/%d.%d] ",i,route.via,fri.zone,fri.net,fri.node,fri.point);
      switch(route.mode)
        {
          case ROUTE_ALL: 
            printf("ALL\n");
            break;
          case ROUTE_ZONE:
            printf("ZONE %d\n",route.zone);
            break;
          case ROUTE_NET:
            printf("NET %d:%d\n",route.zone,route.net);
            break;
          case ROUTE_HUB:
            printf("HUB %d:%d/%d\n",route.zone,route.net,route.node);
            break;
          case ROUTE_NODE:
            printf("NODE %d:%d/%d\n",route.zone,route.net,route.node);
            break;
          case ROUTE_POINT:
            printf("POINT %d:%d/%d.%d\n",route.zone,route.net,route.node,route.point);
            break;
          default:
            puts("??????????????????");
        }
      i++;
    }
  fclose(ff);
}


pickorigin(unsigned int area,int origin)
{
  if ((origin<0) || (origin>255)) {fprintf(stderr,"%d is not a valid origin number",origin); esci("");}
  if ((area<0) || (area>=cfg.pipsize)) {fprintf(stderr,"%d is not a valid area number",area); esci("");}
  if (*areas[area].tag==0) {fprintf(stderr,"area #%d is not defined",area); esci("");}
  if (*areas[area].tag=='#') esci("you cannot define an origin for a local or netmail area");
/*  if (!(*origins[area])) {sprintf(lbuff,"origin #%d is not defined",origin); esci(lbuff);}*/
  areas[area].origin=origin;
  printf("Area #%d(%s) has now origin #%d(%s)\n",area,areas[area].tag,origin,origins[origin]);
  write_areas();
}


resetexport(uint area,uint friend)
{
  if ((friend<0) || (friend>255)) esci("Not a valid friend number");
  if ((area<0) || (area>=cfg.pipsize)) esci ("Not a valid area number");
  resetbit(friend,areas[area].forward_to);
  write_areas();
}


setarealevel(uint area,unsigned char level)
{
  areas[area].readlevel=areas[area].writelevel=level;
  write_areas();
}


setattaches(char *new,char style,char *pack)
{
  int i;
  dirset(cfg.attaches,new);
  switch(toupper(style))
    {
      case 'P': cfg.attachstyle=0; cfg.pack_matrix=1; break;
      case 'F': case 'D': cfg.attachstyle=1; cfg.pack_matrix=0; break;
      case 'B': case 'T': cfg.attachstyle=2; cfg.pack_matrix=1; break;
      default: esci("Unknown mailer!");
    }
  if (pack)
    switch(toupper(*pack))
      {
        case 'P': cfg.pack_matrix=1; break;
        case 'D': cfg.pack_matrix=0; break;
        default: esci("Sorry, byt I understand only PACK-MATRIX and DYNAMIC-PACKING methods");
      }
  write_nodeinfo();
}


setexport(uint area,uint friend)
{
  if ((friend<0) || (friend>255)) esci("Not a valid friend number");
  if ((area<0) || (area>=cfg.pipsize)) esci ("Not a valid area number");
  setbit(friend,areas[area].forward_to);
  write_areas();
}


setfriend(uint n,uint zone,uint net,uint node,uint point,char *packer,char *parms,char *priority)
{
  FRIENDND fri;
  uint i;
  if ((n<0) || (n>255)) esci("Not a valid friend number");
  sprintf(fn,"%sFriendNd.Pip",cfg.pipdir); f=fopen(fn,"rb+");
  if (f==NULL) {printf("Creating %s...\n",fn); f=fopen(fn,"wb");}
  i=filelength(fileno(f))/sizeof(fri);
  if (i>=n)
    fseek(f,n*sizeof(fri),SEEK_SET);
  else
    {
      printf("Adding empty entries");
      fseek(f,i*sizeof(fri),SEEK_SET);
      fri.zone=fri.net=fri.node=fri.point=0; fri.flags=FRI_SET_DEL; /* inactive */
      *fri.notes=*fri.packer=*fri.packerparms=fri.areafix_lev=*fri.areafix_pw=*fri.pack_pw=0;
      while (i<n) {fwrite(&fri,sizeof(fri),1,f); putchar('.'); i++;}
      printf("\n");
    }
  fri.zone=zone; fri.net=net; fri.node=node; fri.point=point;
  strcpy(fri.packer,packer); strcpy(fri.packerparms,parms);
  *fri.notes=*fri.pack_pw=fri.areafix_lev=*fri.areafix_pw=0;
  fri.flags=2; /* active */
  fri.flags|=((*priority=='C')?FRI_SET_CRASH:0)|((*priority=='H')?FRI_SET_HOLD:0);
  fwrite(&fri,sizeof fri,1,f);
  fclose(f);
  printf("Friend#%d set to %d:%d/%d.%d\n",n,zone,net,node,point);
}


setorigin(int i, char *s)
{
  if ((i<0) || (i>255)) {fprintf(stderr,"%d is not valid as origin number",i); esci("");}
  if (strlen(s)>59) {fprintf(stderr,"\"%s\" is too long to be an origin",s); esci("");}
  sprintf(origins[i],"%s",s);
  sprintf(fn,"%sOrigins.Pip",cfg.pipdir);
  if ((f=fopen(fn,"wb"))==NULL) {sprintf(lbuff,"I cannot write origin file! please check for %s",fn); esci(lbuff);}
  fwrite(origins,sizeof origins,1,f); fclose(f);
  printf("Origin#%d set to \"%s\"\n",i,s);
}


setpacker(char *new,char *nparms)
{
  strupr(new);
  if (strlen(new)>12) {fprintf(stderr,"Name too long to be an executable program: complete path is not needed\7\n"); exit(2);}
  if (strlen(new)>39) {fprintf(stderr,"Too many parameters\7\n"); exit(2);}
  if (!strstr(new,".COM") && !(strstr(new,".EXE"))) {fprintf(stderr,"You must specify .EXE or .COM extension"); exit(2);}
  strcpy(cfg.packer,new);
  strcpy(cfg.packerparms,nparms);
  write_nodeinfo();
}


setpurge(uint area,uint nrmsg,uint days)
{
  areas[area].nrmsgs=nrmsg;
  areas[area].days=days;
  write_areas();
}


settag(uint area,char *tag)
{
  strcpy(areas[area].tag,strupr(tag));
  write_areas();
  if(*tag!='#')
    printf("Area #%d has new echotag %s\n",area,tag);
}


settear(char *s)
{
  if(strlen(s)>20) esci("Tearline Too long");
  strcpy(cfg.addtear,s);
  write_nodeinfo();
}


setunpacker(char *new,char *nparms)
{
  strupr(new);
  if (strlen(new)>12) {fprintf(stderr,"Name too long to be an executable program: complete path is not needed\7\n"); exit(2);}
  if (strlen(new)>39) {fprintf(stderr,"Too many parameters\7\n"); exit(2);}
  if (!strstr(new,".COM") && !(strstr(new,".EXE"))) {fprintf(stderr,"You must specify .EXE or .COM extension"); exit(2);}
  strcpy(cfg.jolly,new);
  strcpy(cfg.jollyparms,nparms);
  write_nodeinfo();
}


setuser(int i,char *name)
{
  if ((i<0) || (i>7)) {fprintf(stderr,"%d is not valid as user number",i); esci("");}
  if (strlen(name)>59) {fprintf(stderr,"\"%s\" is too long to be an user name",name); esci("");}
  if(stricmp(name,"NULL"))
    strcpy(cfg.usernames[i],name);
  else
    *cfg.usernames[i]=0;
  write_nodeinfo();
}
